#!/usr/bin/perl -s
# E.Angenault
# V 1.0
# Aot 2011
#
# Definition du traitement 
# + Verification override
# + Verification du statut
#   Si SUCCESS
#     Start et End date
#     Log Application
#     => statut = DONE
#   Si RUNNING
#     Si le job est une boite
#        -> Liste des traitements
#     Si c'est un file watcher
#        -> Fichier attendu
#           * Action : mettre  jour le fichier
#   Si FAILURE
#  	  Afficher les logs application & agent
#     Si c'est une boite
#        -> Liste des jobs
#   Si TERMINATED
#  	  Afficher les logs application & agent
#     Si c'est une boite
#        -> Liste des jobs
#   Si RESWAIT
#     Lister les ressources
#   Si ACTIVATED
#     Afficher box_name & condition
#     Liste l'tat des conditions
#   Si PENDING
#     Afficher la machine
#     Afficher l'tat de la machine
#   Si ON_ICE
#     Lister les dpendances
#     Si c'est une boite
#        -> Liste des jobs
#   Si ON_HOLD
#     Lister les dpendances
#     Si c'est une boite
#        -> Liste des jobs
#   Si DONE
#     Si condition de date 
#      -> Verifier le prochain lancement
#        Si pas de lancement
#         -> Vrifier calendrier et dates
# + Afficher l'audit des derniers changements
# FIN

if (!$job) {
	PrintTitle('E. Angenault                                                Ordonnancement.org',1);
	PrintLine("\n\tSyntaxe:\tautodiag.pl -job=<JOBNAME>\n");
	exit(1);
}

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
$now = sprintf("%04d/%02d/%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min, $sec);

@ColStatus = qw ( FA red TE red SU green PE magenta RU yellow IN white RW magenta AC white OI cyan OH blue ST yellow);
%ColorStatus= @ColStatus;
@SStatus = qw ( RU RUNNING FA FAILURE SU SUCCESS ST STARTING TE TERMINATED RW RESWAIT AC ACTIVATED PE PENDING OI ON_ICE OH ON_HOLD );
%StrStatus= @SStatus;
foreach $k (keys %StrStatus) {
	$v= $StrStatus{$k};
	$RevStatus{$v} = $k;
}

# Si page HTML
if ($output eq 'html') {
	print "<html>
<head>
<title>$job</title>
<!-- Styles -->
<style>
body {
	font-family: arial;
}
tr.red { color: red; }
tr.green { color: green; }
tr.orange { color: orange; }
tr.blue { color: blue; }
tr.cyan { color: cyan; }
tr.white { color: grey; }
tr.yellow { color: black; }
p.red { color: red; }
p.green { color: green; }
p.orange { color: orange; }
p.blue { color: blue; }
p.cyan { color: cyan; }
p.white { color: grey; }
p.yellow { color: black; }
td {
	border-bottom : 1px solid grey;
	border-right : 1px solid grey;
}
th {
	border-bottom : 1px solid grey;
	border-right : 1px solid grey;
}
code {
	background-color: #AAA;
	color: white;
	border: 1px solid black;
}
</style>
</head>
<body>\n";
}

$version = ExecCmd('autoflags -x');
$version =~ s/\n//g;
PrintLine($version,'white');

# Information du traitement
Autorep_definition($job);
PrintTitle("Job : $job",1);
PrintLine("Desc.: ".$Definition{$job}->{'description'});
PrintSchedule;

# Attention a l'override !
Autorep_override($job);

# Detail 
Autorep_detail($job);
$status=$Status{$job}->{'status'};
PrintTitle( "Status",2 );
$st = $Status{$job}->{'st'};
PrintLine($status,$ColorStatus{$st});

if ($status eq 'SUCCESS') {
	PrintArray( 'Start',$Status{$job}->{'start'},
				'End',$Status{$job}->{'end'} );
#	print "More  : ".$Status{$job}->{'more'}."\n";

	# Log Applicative
	PrintAppLog($job);
	
	$status = 'DONE';
}
elsif ($status eq 'RUNNING') {
	if ($Definition{$job}->{'job_type'} eq 'BOX') {
		PrintJobs($job);
		PrintArray( 'Start', $Status{$job}->{'start'} );
	}
	# Est ce un file watcher ?
	elsif ($Definition{$job}->{'job_type'} eq 'FT') {
		PrintArray( 'Start', $Status{$job}->{'start'},
					'Machine', $Status{$job}->{'machine'} );
		PrintWatchFile($job);
		Action_CREATE_FILE($job,$Definition{$job}->{'watch_file'});
	}
	else {
		PrintArray( 'Start', $Status{$job}->{'start'},
					'Machine', $Status{$job}->{'machine'} );
		PrintCommand($job);
	}
}
elsif ($status eq 'FAILURE') {
	# On recupere le run, le retry est l'exit
	($info,$exit) = split(" +",$Status{$job}->{'more'});
	($run,$try) = split('/',$info);
	
	PrintArray( 'Start',$Status{$job}->{'start'},
				'End',$Status{$job}->{'end'},
				'Run', $run,
				'Retry', $try,
				'Exit', $exit );
	PrintHistory($job);
	if ($Definition{$job}->{'job_type'} ne 'BOX') {
		PrintTitle( "Definition", 3);
		PrintArray( 'Machine',$Definition{$job}->{'machine'},
					'Owner',$Definition{$job}->{'owner'},
					'Command',$Definition{$job}->{'command'},
					);
	}
	
	# Log Applicative
	PrintAppLog($job);
	PrintAgtLog($job);
	PrintEnvLog($job);
	
	# Est ce une boite
	PrintJobs($job);

	$status = 'DONE';
}
elsif ($status eq 'TERMINATED') {
	PrintArray( 'Start',$Status{$job}->{'start'},
				'End',$Status{$job}->{'end'} );

	# Log Applicative
	PrintAppLog($job);
	PrintAgtLog($job);
	PrintEnvLog($job);
	
	# Est ce une boite
	PrintJobs($job);

	$status = 'DONE';
}
elsif ($status eq 'RESWAIT') {
	PrintArray( 'Resources', $Definition{$job}->{'resources'} );
	
	# Affichage des ressources
	PrintResources($job);
}
elsif ($status eq 'ACTIVATED') {
	PrintArray ( 'Box name', $Definition{$job}->{'box_name'} );
	
	# Affichage des conditions
	PrintConditions($job);
	# Affichage planification
	PrintNextStart($job);
}
elsif ($status eq 'PENDING') {
	PrintArray( 'Machine', $Definition{$job}->{'machine'} );

	# Affichage des conditions
	PrintMachineStatus($job);
}
elsif ($status eq 'ON_ICE') {
	PrintDependencies($job,$status);
	PrintJobs($job);
}
elsif ($status eq 'ON_HOLD') {
	PrintDependencies($job,$status);
	PrintJobs($job);
}

# Si c'est termin, on fournit les informations de prochains lancement
if ($status eq 'DONE') {
	# Est ce qu'il y a une condition de date ?
	PrintNextStart($job);
	
	# Est ce qu'il est dans une boite ?
	# Est ce qu'il y a un job dclencheur
	
}

# Audit
PrintAudit($job);

# Etat du serveur
PrintServer();

# Fin
if ($output eq 'html') {
	print "<hr/>$now";
	print "</body></html>";
}
else {
	PrintLine('==============================================================================','white');
	PrintLine($now);
}

# Premier tape : Quel est son tat 
sub Autorep_detail {
my($job)=@_;

	$autorep_d = ExecCmd("autorep -J $job -w -d -l0");
	# print $autorep_d;
	foreach (split("\n",$autorep_d)) {
		if (s/^\s*(.*?)  *([\d\/\:\- ]{19})  ([\d\/\:\- ]{19})  (\w\w) (.*?) *//) {
			$Status{$job}->{'status'} = $StrStatus{$4};
			$Status{$job}->{'st'} = $4;
			$Status{$job}->{'start'} = $2;
			$Status{$job}->{'end'} = $3;
			$Status{$job}->{'more'} = $_;
		}
		elsif (/\[\*\*\* (.*?) \*\*\*\]/) {
			# a attacher  la ligne suivante
			$History{$job}->{$n+1}->{'alarm'}  = $1;
		}
		elsif (/\<(.*?)\>/) {
			# a attacher  la ligne prcdente
			$History{$job}->{$n}->{'text'}  = $1;
		}
		elsif (s/^\s*(SUCCESS|FAILURE|STARTING|RUNNING|RESOURCE|MAX_RETRYS|RESTART|\[FORCE_STARTJOB\]|\[STARTJOB\]|JOBFAILURE) *(\d{4}\/\d{2}\/\d{2} \d{2}:\d{2}:\d{2})\s*(\d*)  (\w{2})  (\d{4}\/\d{2}\/\d{2} \d{2}:\d{2}:\d{2})//) {
			$n = $History{$job}->{'lines'};
			$n++;
			$History{$job}->{'lines'}++;
			$History{$job}->{$n}->{'status'} = $1;
			$History{$job}->{$n}->{'timestamp'} = $2;
			$History{$job}->{$n}->{'ntry'} =$3;
			$History{$job}->{$n}->{'processed'} = $4;
			$History{$job}->{$n}->{'more'} = $_;
		} # Version infrieure a 11 
		elsif (s/^\s*(.*?)  *([\d\/\:\- ]{19}) ([\d\/\:\- ]{19})  (\w\w) (.*?) *//) {
			$Status{$job}->{'status'} = $StrStatus{$4};
			$Status{$job}->{'st'} = $4;
			$Status{$job}->{'start'} = $2;
			$Status{$job}->{'end'} = $3;
			$Status{$job}->{'more'} = $_;
		}
		
	}
}

sub Autorep_definition {
my($job)=@_;
	# On part de la definition du job 
	$autorep_q =ExecCmd("autorep -J $job -q -l0");
	foreach (split("\n",$autorep_q)) {
		if (s/^(.*?): //) {
			$f = $1;
			$val = $_;
			if ($f eq "insert_job") {
				s/^.*? (job_type): //;
				s/ *$//;
				$Definition{$job}->{'job_type'} = $_;
			}
			else {
				$Definition{$job}->{$f} = $val;
			}
		}
	}
}

sub Autorep_override {
my($job)=@_;
my(@Diff)=();

	# On part de la definition du job 
	$autorep_q =ExecCmd("autorep -J $job -o0 -l0");
	$ok = 0;
	foreach (split("\n",$autorep_q)) {
		if (/\*CAUAJM_I_10350  Over-Ride #\d*  Set by User: (.*?)  on \[(.*?)\]\*/) {
		}
		elsif (/\*CAUAJM_I_10351  Over-Ride CURRENTLY IN EFFECT.\*/) {
			$ok=1;
		}
		elsif (s/^(.*?): //) {
			$f = $1;
			$val = $_;
			if ($f eq "insert_job") {
				s/^.*? (job_type): //;
				s/ *$//;
				$Override{$job}->{'job_type'} = $_;
			}
			else {
				$Override{$job}->{$f} = $val;
				if (($f ne '') && ($Definition{$job}->{$f}) && ($Definition{$job}->{$f} ne $val)) { 
					push(@Diff,$f);
				}
			}
		}
	}
	return if ($ok==0);
	
	PrintLine('----------');	
	PrintLine("Override",'red');
	foreach $d (@Diff) {
		PrintLine($d,'white');
		PrintLine("\t".$Definition{$job}->{$d},'white');
		PrintLine("->\t".$Override{$job}->{$d},'red');
	}
}

# FONCTION DE PUBLICATION
sub PrintSchedule {
my($job)=@_;

	PrintTitle("Planification",3);
	if ($Definition{$job}->{'date_conditions'}==0) {
		print "No schedule.\n";
	}
	else {
		if ($Definition{$job}->{'days_of_week'}) {
			print "Days of week :\t$Def{'days_of_week'}\n";
			@wday = ('su','mo','tu','we','th','fr','sa','su');
			if ($Definition{$job}->{'days_of_week'} =~ /all/) {
				print "\tStart every day => OK";
			}
			else {
				$today = $Now[6];
				$str = $wday[$today];
				print "\tNow '$str' => ";
				if ($Definition{$job}->{'days_of_week'} =~ /$str/) {
					print "OK";
				}
				else {
					print "KO !";
				}
				print "\n";
			}
		}
		if ($Definition{$job}->{'start_times'} eq "") {
			print "No start time => Start time at midnight\n";
		}
		else {
			print "Start time :\t$Def{'start_times'}\n";
		}
	}
}

sub PrintHistory {
my($job)=@_;

	PrintTitle('History',3);
	$n = $History{$job}->{'lines'};
	for ($i=$n;$i>0;$i--) {
		if ($History{$job}->{$i}->{'processed'} eq 'PD') {
			$status = $History{$job}->{$i}->{'status'};
			if ($History{$job}->{$i}->{'alarm'} ne '') {
				$color='magenta';
			}
			else {
				$st = $RevStatus{$status};
				$color = $ColorStatus{$st};
			}
			PrintLine($History{$job}->{$i}->{'timestamp'}.' '.$History{$job}->{$i}->{'alarm'},$color);
			if ($status =~ /\[/) {
				PrintLine(substr('                     ACTION : ',-13).$History{$job}->{$i}->{'status'},$color);				
			}
			else {
				PrintLine(substr('                     EVENT : ',-13).$History{$job}->{$i}->{'status'},$color);				
			}
			
			if ($History{$job}->{$i}->{'text'} ne '') {
				PrintLine(substr('                     ',-13).$History{$job}->{$i}->{'text'},$color);
			}
			# on sort si c'est une action
			if ($status =~/\[/) {
				return;
			}
		}
	}
}

# Traitements inclu dans le job
sub PrintJobs {
my($job)=@_;

	# Inutile si ce n'est pas une boite
	return unless ($Definition{$job}->{'job_type'} eq 'BOX');
	PrintTitle('Jobs in the box',3);
	$autorep_d =ExecCmd("autorep -J $job -w");
	print '<table>' if ($output eq 'html');
	foreach (split("\n",$autorep_d)) {
		if (s/^\s*(.*?)  *([\d\/\:\- ]{19})  ([\d\/\:\- ]{19})  (\w\w) (.*?) *//) {
			$job = $1;
			$status = $4;
			$start = $2;
			$end = $3;
			$more = $_;
			$st = $StrStatus{$status};
			$color = $ColorStatus{$status};
			if ($output eq 'html') {
				print "<tr class=\"$color\"><td>$job</td><td>$status</td><td>$start</td><td>$end</td></tr>";
			}
			else {
				if (length($job)>35) {
					PrintLine($job,$color);
					PrintLine("                                    ".$status." $start $end",$color);
				}
				else {
					PrintLine(substr("$job                                        ",0,35)." ".$status." $start $end",$color);
				}
			}
		}
	}
	print '</table>' if ($output eq 'html');
}

# Traitements inclu dans le job
sub PrintResources {
my($job)=@_;

	# Inutile si ce n'est pas une boite
	$res = $Definition{$job}->{'resources'};
	PrintHR();
	PrintLine("Resource                         Type     Def.    Disp. Machine");
	while ($res =~ s/\((.*?),//) {
		$r = $1;
# Ex GLOBAL_CHECK_Depletable                                                                                                          D      ---                                                                              0       0
		$autorep =ExecCmd("autorep -V $r");
		foreach (split("\n",$autorep)) {
			($Resource,$Type,$Machine,$Defined,$Available) = split(" +");
			if ($Resource eq $r) {
				if ($Available>0) {
					$color= 'green';
				}
				else {
					$color= 'red';
				}
				PrintLine(substr("$Resource                                   ",0,32)." ".substr("    $Type",-4)." ".substr("        $Defined",-8)." ".substr("        $Available",-8)." ".$Machine,$color);
			}
		}		
	}
}

# Traitements inclu dans le job
sub PrintConditions {
my($job)=@_;

	# Inutile si ce n'est pas une boite
	$cnd = $Definition{$job}->{'condition'};
	if (!$cnd) {
		PrintLine("Pas de condition.", 'green');
		return;
	}
	PrintHR();
	PrintLine("condition : ".$Definition{$job}->{'condition'},'yellow');
	
	while (($cnd =~ s/([sdf])\((.*?)\)//) || ($cnd =~ s/(v)\((.*?)\)(\s+=\s+[\w"]*)//)) {
		$t = $1; $j = $2; $more = $3;
		$color = 'red';
		if ($t eq 'v') {
			$type = 'VAR';
			$autostatus =ExecCmd("autostatus -G $j");
			if ($autostatus eq '') {
				$status = "INCONNU";
			}
			else {
				$autostatus =~ s/\n//;
				$color='green' if ("\"$autostatus\" $more");
			#	print eval { $autostatus $more };
			}
		}
		else {
			$type = 'JOB';
			$autostatus =ExecCmd("autostatus -J $j");
			$autostatus =~ s/\n//;
			if ($autostatus eq '') {
				$status = "INCONNU";
			}
			else {
				$autostatus =~ s/\n//;
				if (
					(($t eq 's') && ($autosys eq 'SUCCESS'))
					|| (($t eq 'f') && ($autosys eq 'FAILURE'))
					|| (($t eq 'd') && ( ($autosys eq 'FAILURE') || ($autosys eq 'SUCCESS'))) 
					) {
					$color='green';
				}
			}
		}
		PrintLine($type." ".substr("$j                                                                  ",0,48)." ".substr("$autostatus                                 ",0,20)." ".$date,$color);
	}
}

sub PrintDependencies {
my($job,$status)=@_;

	PrintTitle('Dependencies', 3);
	$job_depends = ExecCmd("job_depends -J $job -c");
	if ($job_depends eq '') {
		PrintLine("No blocked job.",'green');
	}
	else {
		if ($status eq 'ON_ICE') {
			PrintLine("Following jobs lost constraints :");
		}
		else {
			PrintLine("Following jobs are blocked :");
		}
	}
	my($n)=0;
	foreach ( split("\n",$job_depends) ) {
		if (/^   (.*?) \s*(DONE|SUCCESS|FAILURE|TERMINATED)\($job\)/) {
			$j = $1; $s = $2;
			$s = 'SUCCESS|FAILURE|TERMINATED' if ($s eq 'DONE');
			PrintLine(substr("$j                                                               ",0,52).$s,'white');
			$n++;
		}
	}
	PrintLine("No job",'green') if ($n==0);	
}

sub PrintMachineStatus {
my($job)=@_;

	# Inutile si ce n'est pas une boite
	$mac = $Definition{$job}->{'machine'};
	PrintHR();
	PrintLine("Machine                  Max Load   Current Load  Factor   O/S         Status");
	$autorep =ExecCmd("autorep -m $mac");
	foreach (split("\n",$autorep)) {
		if (s/^${mac} *//) {
			if (/^(.{10}) (.{10}) (.{6}) (.{14}) (.*)/) {
				$Max =$1; $Current =$2; $Factor=$3; $OS=$4; $Status=$5;
			#print "($Max)($Current)($Factor)($OS)($Status)";
			if ($Status eq 'Offline') {
				$color='red';
			}
			else {
				$color='green';
			}
			PrintLine(substr("$mac                          ",0,24)." ".substr("          $Max",-10)." ".substr("          $Current",-10)." ".substr("        $Factor",-8)." ".substr("               $OS",-14)." ".$Status,$color);
			}
		}
	}	
}

sub PrintWatchFile {
my($job)=@_;

	PrintTitle('Watching file',3);
	PrintArray( 'watch_file', $Definition{$job}->{'watch_file'},
				'watch_file_recursive', $Definition{$job}->{'watch_file_recursive'},
				'watch_file_type', $Definition{$job}->{'watch_file_type'},
				'watch_no_change', $Definition{$job}->{'watch_no_change'},
				'continuous', $Definition{$job}->{'continuous'},
				);
}

sub PrintCommand {
my($job)=@_;

	PrintTitle('Command',3);
	PrintArray( 'command', $Definition{$job}->{'command'},
				'machine', $Definition{$job}->{'machine'},
				'owner', $Definition{$job}->{'owner'}
				);
}

sub PrintAppLog {
my($job)=@_;

	# Pas de log applicatif si c'est une boite
	return if ($Definition{$job}->{'job_type'} eq 'BOX');

	PrintTitle('Application log',2);

	if ($Definition{$job}->{'std_out_file'} ne '') {	
		$autotrack =ExecCmd("autosyslog -J $job -t O");
		PrintLine($Definition{$job}->{'std_out_file'});
		print '<pre>' if ($output eq 'html');
		PrintLine($autotrack, 'green');
		print '</pre>' if ($output eq 'html');
	}
	
	if ( ($Definition{$job}->{'std_err_file'} ne '')
		&& ($Definition{$job}->{'std_err_file'} ne $Definition{$job}->{'std_out_file'}) ) {	
		PrintTitle('STDERR',3);	
		$autotrack =ExecCmd("autosyslog -J $job -t E");
		PrintLine($Definition{$job}->{'std_err_file'});
		print '<pre>' if ($output eq 'html');
		PrintLine($autotrack, 'red');
		print '</pre>' if ($output eq 'html');
	}
}

sub PrintAgtLog {
my($job)=@_;

	# Pas de log agent si c'est une boite
	return if ($Definition{$job}->{'job_type'} eq 'BOX');

	PrintTitle( 'Log agent', 2);
	$autotrack =ExecCmd("autosyslog -J $job -t A");
	print '<pre>' if ($output eq 'html');
	PrintLine($autotrack);
	print '</pre>' if ($output eq 'html');
}

sub PrintEnvLog {
my($job)=@_;

	# A voir avec CA, l'intrt de -t P
	return;
	
	# Pas de log profile si c'est une boite
	return if ($Definition{$job}->{'job_type'} eq 'BOX');

	PrintTitle('Log Profile',2);
	$autotrack =ExecCmd("autosyslog -J $job -t P");
	print '<pre>' if ($output eq 'html');
	PrintLine($autotrack);
	print '</pre>' if ($output eq 'html');
}

sub PrintAudit {
my($job)=@_;

	PrintTitle('Audit', 2);	
	$autotrack = ExecCmd("autotrack -J $job -v")."::::::::::::::\n";
	my($n)=1;
	print '<table>' if ($output eq 'html');
	foreach (split("\n",$autotrack)) {
		if (/^::::/) {
			if ($n>0) {
				if ($code =~ /Job definition change/) {
					$color = 'red';
				}
				elsif ($code =~ /Sendevent issued/) {
					$color = 'yellow';
				}
				else {
					$color = 'white';
				}
				if ($output eq 'html') {
					print "<tr class=\"$color\"><td>$date</td><td>$user</td><td>".join(', ',@Info)."</td></tr>";
				}
				else {
					PrintLine($date." ".substr("$user                                    ",0,30).substr("$code                                                  ",0,30),$color);
					PrintLine("\t".join(', ',@Info),$color);
				}
			}
			@Info = ();
			$n=0;
		}
		if ($n==1) {
			$user = $_;
		}
		if ($n==2) {
			$date = $_;
		}
		if (s/^CAUAJM_I_\d* //) {
			$code = $_;
		}
		if (s/^  command: //) {
			s/\"(\-\w)\"/$1/g;
			push(@Info,$_);
		}
		if (s/^\* //) {
			push(@Info,$_);
		}		
		$n++;
	}
	print '</table>' if ($output eq 'html');
}

sub PrintServer {
	PrintTitle( 'Check server', 2 );
	
	$autosyslog = ExecCmd("autosyslog -e -l 20");
	print '<table>' if ($output eq 'html');
	foreach (split("\n",$autosyslog)) {
		# [2011/08/12 16:13:01]      CAUAJM_I_10082 [parvl9011657 connected for PAUTOSYSP01-C1CHK-Chase 184.28313.1]
		if (s/^\[(.*?)\]\s*//) {
			$date = $1; $log = $_;
			$log =~ s/^CAUAJM_I_\d{5}//;
			$log =~ / STATUS: (.*?) /;
			$color = $1;
			if ($color) {
				$r = $RevStatus{$color};
				$color = 'white' unless ($color=$ColorStatus{$r});
			} 
			else {
				$color = 'magenta';
			}
			
			if ($output eq 'html') {
				print "<tr class=\"$color\"><td>$date</td><td>$log</td></tr>";
			}
			else {
				$log =~ s/ (EVENT|STATUS|MACHINE|JOB|TEXT|EXITCODE|ALARM): /\n\t$1\t/g;
				PrintLine($date.' '.$log,$color);
			}
		}
	}
	print '</table>' if ($output eq 'html');
	# Comparer dernier date avec l'heure	
}

sub PrintNextStart {
my($job)=@_;

	# Si pas de condition de date, inutile
	return unless ($Definition{$job}->{'date_conditions'}==1);

	PrintTitle('Next start', 3);
	
	# On recherche un evenement UP pour un start_job
	$autorep_d = ExecCmd("autorep -J $job -w -d -l0");
	# print $autorep_d;
	my($next) = '';
	foreach (split("\n",$autorep_d)) {
		if (/^\s*\[STARTJOB\]\s*(\d{4}\/\d{2}\/\d{2} \d{2}:\d{2}:\d{2})\s*(\d*)  (UP)/) {
			PrintLine($1,'green');
			return;
		}
	}
	
	PrintLine( "No schedule !",'red');
	if ($Definition{$job}->{'run_calendar'}) {
		$calendar = $Definition{$job}->{'run_calendar'};
		PrintTitle("Calendar $calendar", 3);
		# Verification 
		ExecCmd("autocal_asc -s $calendar -E autodiag.tmp");
		$autocal = GetFile("autodiag.tmp");
		if ($autocal eq '') {
			PrintLine("\t$calendar doesn't exist !",'red');
		}
		$n=0; $ok=0;
		my(@Dates) = ();
		foreach (split("\n",$autocal)) {
			if ($n>0) {
				$date = USDate2ISO($_);
				if (($ok==0) && (DateSup($date,$now)==1)) {
					$next = $date;
					$ok = 1;
				}
				push(@Dates,$date);
			}
			$n++;
		}
		PrintLine(join(", ",@Dates),'white');
		if ($ok ==1) {
			PrintLine("Next date : $next",'green');
			Action_RESCHEDULE($job);
		}
		else {
			PrintLine("No date.",'red');
		}
	}
}

sub ExecCmd {
($cmd)=@_;
	# On verifie que les executables existent
	if ($version !~ /^1\d\./) {
		return if ($cmd =~ /^(autosyslog)/);
	}
	
	# mode verbeux pour avoir les commandes passes
	if ($verbose eq 'y') {
		if ($output eq 'html') {
			print( "<code>$cmd</code><br/>");	
		}
		else {
			PrintLine('-Commande Autosys---------','white');	
			PrintLine($cmd, 'yellow');	
			PrintLine('----------');	
		}
	}
return `$cmd`;
}

sub GetFile {
my($file)=@_;

	open(FILE,$file) || die ("Tempfile empty !\n");
	$content ='';
	while (<FILE>) {
		$content .= $_;
	}
	close(FILE);
return $content;
}

sub USDate2ISO {
my($d) = @_;
	if ($d =~ /^(\d\d)\/(\d\d)\/(\d\d\d\d) (\d\d):(\d\d)/) {
		return "$3/$1/$2 $4:$5:00";
	}
return;
}

sub DateSup {
my($d1,$d2) = @_;
	$d1 =~ s/[\/: ]//g;
	$d2 =~ s/[\/: ]//g;
	return 1 if ($d1 > $d2);
return 0;
}

# Actions conseilles
sub Action_CREATE_FILE {
($job,$file) = @_;
	return if ($action eq 'n');
	
	PrintTitle('Available actions',2);

	PrintLine("* Create file $file", 'white');
	PrintLine("echo ok > $file", 'cyan');

	PrintLine("* Kill job $job", 'white');
	PrintLine("sendevent -E KILLJOB -J $job", 'cyan');
	
	PrintLine("* Change status to OK", 'white');
	PrintLine("sendevent -E CHANGE_STATUS -J $job -s SUCCESS", 'cyan');

}

# Actions conseilles
sub Action_RESCHEDULE {
($job) = @_;
	return if ($action eq 'n');
	
	PrintTitle('Available actions',2);

	PrintLine("* Reschedule job $job", 'white');
	PrintLine("echo update_job: $job | jil", 'cyan');

}

sub PrintLineVT {
my($line,$color) = @_;
@Col = ( 'green', "\033[0;32;40m", 'red', "\033[0;31;40m", 'white', "\033[0;37;40m", 'yellow', "\033[0;33;40m", 'blue', "\033[0;34;40m", 'magenta', "\033[0;35;40m",  'cyan', "\033[0;36;40m");
%Color= @Col;
	$color = 'white' unless $color;
	print $Color{$color}."$line\n";
}

sub PrintLineHTML {
my($line,$color) = @_;

	print "<p class=\"$color\">$line</p>\n";
}

sub PrintLine {
	if ($output eq 'html') {
		PrintLineHTML(@_);
	}
	else {
		PrintLineVT(@_);
	}
}

sub PrintTitle {
my($line,$level) = @_;

	if ($output eq 'html') {
		print "<h$level>$line</h$level>\n";	
	}
	else {
		if ($level==1) {
			PrintLine('==============================================================================');
			PrintLineVT($line,'yellow');
			PrintLine('------------------------------------------------------------------------------');
			print 
		}
		elsif ($level==2) {
			PrintLine("\n$line",'white');
			PrintLine(substr('==================================',0,length($line)));
		}
		elsif ($level==3) {
			PrintLine("\n$line",'white');
			PrintLine(substr('--------------------------------------',0,length($line)));
		}
		else {
			PrintLineVT("\n$line",'white');
		}
	}
}

sub PrintArray {
my(@Val) = @_;
%Values = @Val;
	if ($output eq 'html') {
		print '<table>';
		foreach $k (keys %Values) {
			print "<tr><th>$k</th><td>".$Values{$k}."</td></tr>";
		}
		print '</table>';
	}
	else {
		foreach $k (keys %Values) {
			print "\033[0;37;40m".substr("                         $k : ",-25);
			PrintLine( $Values{$k}, 'yellow' );
		}
	}
}

sub PrintHR {

	if ($output eq 'html') {
		print "</hr>";
	}
	else {
		PrintLine('------------------------------------------------------------------------------','white');
	}
}
